# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from contextvars import ContextVar, Token

from opentelemetry.context.context import Context, _RuntimeContext


class ContextVarsRuntimeContext(_RuntimeContext):
    """An implementation of the RuntimeContext interface which wraps ContextVar under
    the hood. This is the preferred implementation for usage with Python 3.5+
    """

    _CONTEXT_KEY = "current_context"

    def __init__(self) -> None:
        self._current_context = ContextVar(
            self._CONTEXT_KEY, default=Context()
        )

    def attach(self, context: Context) -> Token[Context]:
        """Sets the current `Context` object. Returns a
        token that can be used to reset to the previous `Context`.

        Args:
            context: The Context to set.
        """
        return self._current_context.set(context)

    def get_current(self) -> Context:
        """Returns the current `Context` object."""
        return self._current_context.get()

    def detach(self, token: Token[Context]) -> None:
        """Resets Context to a previous value

        Args:
            token: A reference to a previous Context.
        """
        self._current_context.reset(token)


__all__ = ["ContextVarsRuntimeContext"]



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Mapping, Optional, Sequence, Tuple, Union

# This is the implementation of the "Any" type as specified by the specifications of OpenTelemetry data model for logs.
# For more details, refer to the OTel specification:
# https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/logs/data-model.md#type-any
AnyValue = Union[
    str,
    bool,
    int,
    float,
    bytes,
    Sequence["AnyValue"],
    Mapping[str, "AnyValue"],
    None,
]

AttributeValue = Union[
    str,
    bool,
    int,
    float,
    Sequence[str],
    Sequence[bool],
    Sequence[int],
    Sequence[float],
]
Attributes = Optional[Mapping[str, AttributeValue]]
AttributesAsKey = Tuple[
    Tuple[
        str,
        Union[
            str,
            bool,
            int,
            float,
            Tuple[Optional[str], ...],
            Tuple[Optional[bool], ...],
            Tuple[Optional[int], ...],
            Tuple[Optional[float], ...],
        ],
    ],
    ...,
]



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase

from opentelemetry.metrics import Observation


class TestObservation(TestCase):
    def test_measurement_init(self):
        try:
            # int
            Observation(321, {"hello": "world"})

            # float
            Observation(321.321, {"hello": "world"})
        except Exception:  # pylint: disable=broad-exception-caught
            self.fail(
                "Unexpected exception raised when instantiating Observation"
            )

    def test_measurement_equality(self):
        self.assertEqual(
            Observation(321, {"hello": "world"}),
            Observation(321, {"hello": "world"}),
        )

        self.assertNotEqual(
            Observation(321, {"hello": "world"}),
            Observation(321.321, {"hello": "world"}),
        )
        self.assertNotEqual(
            Observation(321, {"baz": "world"}),
            Observation(321, {"hello": "world"}),
        )



# pylint: disable=W0212,W0222,W0221
import typing
import unittest

import opentelemetry._events as events
from opentelemetry.test.globals_test import EventsGlobalsTest
from opentelemetry.util.types import Attributes


class TestProvider(events.NoOpEventLoggerProvider):
    def get_event_logger(
        self,
        name: str,
        version: typing.Optional[str] = None,
        schema_url: typing.Optional[str] = None,
        attributes: typing.Optional[Attributes] = None,
    ) -> events.EventLogger:
        return LoggerTest(name)


class LoggerTest(events.NoOpEventLogger):
    def emit(self, event: events.Event) -> None:
        pass


class TestProxy(EventsGlobalsTest, unittest.TestCase):
    def test_proxy_logger(self):
        provider = events.get_event_logger_provider()
        # proxy provider
        self.assertIsInstance(provider, events.ProxyEventLoggerProvider)

        # provider returns proxy logger
        event_logger = provider.get_event_logger("proxy-test")
        self.assertIsInstance(event_logger, events.ProxyEventLogger)

        # set a real provider
        events.set_event_logger_provider(TestProvider())

        # get_logger_provider() now returns the real provider
        self.assertIsInstance(events.get_event_logger_provider(), TestProvider)

        # logger provider now returns real instance
        self.assertIsInstance(
            events.get_event_logger_provider().get_event_logger("fresh"),
            LoggerTest,
        )

        # references to the old provider still work but return real logger now
        real_logger = provider.get_event_logger("proxy-test")
        self.assertIsInstance(real_logger, LoggerTest)



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# type: ignore

# NOTE: The tests in this file are intended to test the semver compatibility of the public API.
# Any tests that fail here indicate that the public API has changed in a way that is not backwards compatible.
# Either bump the major version of the API, or make the necessary changes to the API to remain semver compatible.

# pylint: disable=useless-parent-delegation,arguments-differ

from typing import Optional

from opentelemetry.metrics import (
    Asynchronous,
    Counter,
    Histogram,
    Instrument,
    Meter,
    MeterProvider,
    ObservableCounter,
    ObservableGauge,
    ObservableUpDownCounter,
    Synchronous,
    UpDownCounter,
    _Gauge,
)


class MeterProviderImplTest(MeterProvider):
    def get_meter(
        self,
        name: str,
        version: Optional[str] = None,
        schema_url: Optional[str] = None,
    ) -> Meter:
        return super().get_meter(name, version, schema_url)


def test_meter_provider_subclass_instantiation():
    meter_provider = MeterProviderImplTest()
    assert isinstance(meter_provider, MeterProvider)


class MeterImplTest(Meter):
    def create_counter(self, name, description, **kwargs):
        pass

    def create_up_down_counter(self, name, description, **kwargs):
        pass

    def create_observable_counter(self, name, description, **kwargs):
        pass

    def create_histogram(self, name, description, **kwargs):
        pass

    def create_observable_gauge(self, name, description, **kwargs):
        pass

    def create_observable_up_down_counter(self, name, description, **kwargs):
        pass


def test_meter_subclass_instantiation():
    meter = MeterImplTest("subclass_test")
    assert isinstance(meter, Meter)


class SynchronousImplTest(Synchronous):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)


def test_synchronous_subclass_instantiation():
    synchronous = SynchronousImplTest("subclass_test")
    assert isinstance(synchronous, Synchronous)


class AsynchronousImplTest(Asynchronous):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)


def test_asynchronous_subclass_instantiation():
    asynchronous = AsynchronousImplTest("subclass_test")
    assert isinstance(asynchronous, Asynchronous)


class CounterImplTest(Counter):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)

    def add(self, amount: int, **kwargs):
        pass


def test_counter_subclass_instantiation():
    counter = CounterImplTest("subclass_test")
    assert isinstance(counter, Counter)


class UpDownCounterImplTest(UpDownCounter):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)

    def add(self, amount: int, **kwargs):
        pass


def test_up_down_counter_subclass_instantiation():
    up_down_counter = UpDownCounterImplTest("subclass_test")
    assert isinstance(up_down_counter, UpDownCounter)


class ObservableCounterImplTest(ObservableCounter):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)


def test_observable_counter_subclass_instantiation():
    observable_counter = ObservableCounterImplTest("subclass_test")
    assert isinstance(observable_counter, ObservableCounter)


class HistogramImplTest(Histogram):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)

    def record(self, amount: int, **kwargs):
        pass


def test_histogram_subclass_instantiation():
    histogram = HistogramImplTest("subclass_test")
    assert isinstance(histogram, Histogram)


class GaugeImplTest(_Gauge):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)

    def set(self, amount: int, **kwargs):
        pass


def test_gauge_subclass_instantiation():
    gauge = GaugeImplTest("subclass_test")
    assert isinstance(gauge, _Gauge)


class InstrumentImplTest(Instrument):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)


def test_instrument_subclass_instantiation():
    instrument = InstrumentImplTest("subclass_test")
    assert isinstance(instrument, Instrument)


class ObservableGaugeImplTest(ObservableGauge):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)


def test_observable_gauge_subclass_instantiation():
    observable_gauge = ObservableGaugeImplTest("subclass_test")
    assert isinstance(observable_gauge, ObservableGauge)


class ObservableUpDownCounterImplTest(ObservableUpDownCounter):
    def __init__(
        self, name: str, unit: str = "", description: str = ""
    ) -> None:
        super().__init__(name, unit, description)


def test_observable_up_down_counter_subclass_instantiation():
    observable_up_down_counter = ObservableUpDownCounterImplTest(
        "subclass_test"
    )
    assert isinstance(observable_up_down_counter, ObservableUpDownCounter)



import unittest

from opentelemetry._events import Event


class TestEvent(unittest.TestCase):
    def test_event(self):
        event = Event("example", 123, attributes={"key": "value"})
        self.assertEqual(event.name, "example")
        self.assertEqual(event.timestamp, 123)
        self.assertEqual(
            event.attributes, {"key": "value", "event.name": "example"}
        )

    def test_event_name_copied_in_attributes(self):
        event = Event("name", 123)
        self.assertEqual(event.attributes, {"event.name": "name"})

    def test_event_name_has_precedence_over_attributes(self):
        event = Event("name", 123, attributes={"event.name": "attr value"})
        self.assertEqual(event.attributes, {"event.name": "name"})



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opentelemetry.test.concurrency_test import ConcurrencyTestBase, MockFunc
from opentelemetry.util._once import Once


class TestOnce(ConcurrencyTestBase):
    def test_once_single_thread(self):
        once_func = MockFunc()
        once = Once()

        self.assertEqual(once_func.call_count, 0)

        # first call should run
        called = once.do_once(once_func)  # type: ignore[reportArgumentType]
        self.assertTrue(called)
        self.assertEqual(once_func.call_count, 1)

        # subsequent calls do nothing
        called = once.do_once(once_func)  # type: ignore[reportArgumentType]
        self.assertFalse(called)
        self.assertEqual(once_func.call_count, 1)

    def test_once_many_threads(self):
        once_func = MockFunc()
        once = Once()

        def run_concurrently() -> bool:
            return once.do_once(once_func)  # type: ignore[reportArgumentType]

        results = self.run_with_many_threads(run_concurrently, num_threads=100)

        self.assertEqual(once_func.call_count, 1)

        # check that only one of the threads got True
        self.assertEqual(results.count(True), 1)



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import contextlib
import functools
from typing import TYPE_CHECKING, Callable, Generic, Iterator, TypeVar

V = TypeVar("V")
R = TypeVar("R")  # Return type
Pargs = TypeVar("Pargs")  # Generic type for arguments
Pkwargs = TypeVar("Pkwargs")  # Generic type for arguments

# We don't actually depend on typing_extensions but we can use it in CI with this conditional
# import. ParamSpec can be imported directly from typing after python 3.9 is dropped
# https://peps.python.org/pep-0612/.
if TYPE_CHECKING:
    from typing_extensions import ParamSpec

    P = ParamSpec("P")  # Generic type for all arguments


class _AgnosticContextManager(
    contextlib._GeneratorContextManager,  # type: ignore  # FIXME use contextlib._GeneratorContextManager[R] when we drop the python 3.8 support
    Generic[R],
):  # pylint: disable=protected-access
    """Context manager that can decorate both async and sync functions.

    This is an overridden version of the contextlib._GeneratorContextManager
    class that will decorate async functions with an async context manager
    to end the span AFTER the entire async function coroutine finishes.

    Else it will report near zero spans durations for async functions.

    We are overriding the contextlib._GeneratorContextManager class as
    reimplementing it is a lot of code to maintain and this class (even if it's
    marked as protected) doesn't seems like to be evolving a lot.

    For more information, see:
    https://github.com/open-telemetry/opentelemetry-python/pull/3633
    """

    def __enter__(self) -> R:
        """Reimplementing __enter__ to avoid the type error.

        The original __enter__ method returns Any type, but we want to return R.
        """
        del self.args, self.kwds, self.func  # type: ignore
        try:
            return next(self.gen)  # type: ignore
        except StopIteration:
            raise RuntimeError("generator didn't yield") from None

    def __call__(self, func: V) -> V:  # pyright: ignore [reportIncompatibleMethodOverride]
        if asyncio.iscoroutinefunction(func):

            @functools.wraps(func)  # type: ignore
            async def async_wrapper(*args: Pargs, **kwargs: Pkwargs) -> R:  # pyright: ignore [reportInvalidTypeVarUse]
                with self._recreate_cm():  # type: ignore
                    return await func(*args, **kwargs)  # type: ignore

            return async_wrapper  # type: ignore
        return super().__call__(func)  # type: ignore


def _agnosticcontextmanager(
    func: "Callable[P, Iterator[R]]",
) -> "Callable[P, _AgnosticContextManager[R]]":
    @functools.wraps(func)
    def helper(*args: Pargs, **kwargs: Pkwargs) -> _AgnosticContextManager[R]:  # pyright: ignore [reportInvalidTypeVarUse]
        return _AgnosticContextManager(func, args, kwargs)  # pyright: ignore [reportArgumentType]

    # Ignoring the type to keep the original signature of the function
    return helper  # type: ignore[return-value]



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=W0212,W0222,W0221
import typing
import unittest

import opentelemetry._logs._internal as _logs_internal
from opentelemetry import _logs
from opentelemetry.test.globals_test import LoggingGlobalsTest
from opentelemetry.util.types import Attributes


class TestProvider(_logs.NoOpLoggerProvider):
    def get_logger(
        self,
        name: str,
        version: typing.Optional[str] = None,
        schema_url: typing.Optional[str] = None,
        attributes: typing.Optional[Attributes] = None,
    ) -> _logs.Logger:
        return LoggerTest(name)


class LoggerTest(_logs.NoOpLogger):
    def emit(self, record: _logs.LogRecord) -> None:
        pass


class TestProxy(LoggingGlobalsTest, unittest.TestCase):
    def test_proxy_logger(self):
        provider = _logs.get_logger_provider()
        # proxy provider
        self.assertIsInstance(provider, _logs_internal.ProxyLoggerProvider)

        # provider returns proxy logger
        logger = provider.get_logger("proxy-test")
        self.assertIsInstance(logger, _logs_internal.ProxyLogger)

        # set a real provider
        _logs.set_logger_provider(TestProvider())

        # get_logger_provider() now returns the real provider
        self.assertIsInstance(_logs.get_logger_provider(), TestProvider)

        # logger provider now returns real instance
        self.assertIsInstance(
            _logs.get_logger_provider().get_logger("fresh"), LoggerTest
        )

        # references to the old provider still work but return real logger now
        real_logger = provider.get_logger("proxy-test")
        self.assertIsInstance(real_logger, LoggerTest)



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest.mock import patch

from opentelemetry._logs import LogRecord

OBSERVED_TIMESTAMP = "OBSERVED_TIMESTAMP"


class TestLogRecord(unittest.TestCase):
    @patch("opentelemetry._logs._internal.time_ns")
    def test_log_record_observed_timestamp_default(self, time_ns_mock):  # type: ignore
        time_ns_mock.return_value = OBSERVED_TIMESTAMP
        self.assertEqual(LogRecord().observed_timestamp, OBSERVED_TIMESTAMP)



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# type: ignore

from logging import WARNING
from unittest import TestCase
from unittest.mock import Mock, patch

from opentelemetry.baggage import get_all, set_baggage
from opentelemetry.baggage.propagation import (
    W3CBaggagePropagator,
    _format_baggage,
)
from opentelemetry.context import get_current


class TestW3CBaggagePropagator(TestCase):
    # pylint: disable=protected-access
    # pylint: disable=too-many-public-methods
    def setUp(self):
        self.propagator = W3CBaggagePropagator()

    def _extract(self, header_value):
        """Test helper"""
        header = {"baggage": [header_value]}
        return get_all(self.propagator.extract(header))

    def _inject(self, values):
        """Test helper"""
        ctx = get_current()
        for k, v in values.items():  # pylint: disable=invalid-name
            ctx = set_baggage(k, v, context=ctx)
        output = {}
        self.propagator.inject(output, context=ctx)
        return output.get("baggage")

    def test_no_context_header(self):
        baggage_entries = get_all(self.propagator.extract({}))
        self.assertEqual(baggage_entries, {})

    def test_empty_context_header(self):
        header = ""
        self.assertEqual(self._extract(header), {})

    def test_valid_header(self):
        header = "key1=val1,key2=val2"
        expected = {"key1": "val1", "key2": "val2"}
        self.assertEqual(self._extract(header), expected)

    def test_invalid_header_with_space(self):
        header = "key1 =   val1,  key2 =val2   "
        self.assertEqual(self._extract(header), {})

    def test_valid_header_with_properties(self):
        header = "key1=val1,key2=val2;prop=1;prop2;prop3=2"
        expected = {"key1": "val1", "key2": "val2;prop=1;prop2;prop3=2"}
        self.assertEqual(self._extract(header), expected)

    def test_valid_header_with_url_escaped_values(self):
        header = "key1=val1,key2=val2%3Aval3,key3=val4%40%23%24val5"
        expected = {
            "key1": "val1",
            "key2": "val2:val3",
            "key3": "val4@#$val5",
        }
        self.assertEqual(self._extract(header), expected)

    def test_header_with_invalid_value(self):
        header = "key1=val1,key2=val2,a,val3"
        with self.assertLogs(level=WARNING) as warning:
            self._extract(header)
            self.assertIn(
                "Baggage list-member `a` doesn't match the format",
                warning.output[0],
            )

    def test_valid_header_with_empty_value(self):
        header = "key1=,key2=val2"
        expected = {"key1": "", "key2": "val2"}
        self.assertEqual(self._extract(header), expected)

    def test_invalid_header(self):
        self.assertEqual(self._extract("header1"), {})
        self.assertEqual(self._extract(" = "), {})

    def test_header_too_long(self):
        long_value = "s" * (W3CBaggagePropagator._MAX_HEADER_LENGTH + 1)
        header = f"key1={long_value}"
        expected = {}
        self.assertEqual(self._extract(header), expected)

    def test_header_contains_too_many_entries(self):
        header = ",".join(
            [f"key{k}=val" for k in range(W3CBaggagePropagator._MAX_PAIRS + 1)]
        )
        self.assertEqual(
            len(self._extract(header)), W3CBaggagePropagator._MAX_PAIRS
        )

    def test_header_contains_pair_too_long(self):
        long_value = "s" * (W3CBaggagePropagator._MAX_PAIR_LENGTH + 1)
        header = f"key1=value1,key2={long_value},key3=value3"
        expected = {"key1": "value1", "key3": "value3"}
        with self.assertLogs(level=WARNING) as warning:
            self.assertEqual(self._extract(header), expected)
            self.assertIn(
                "exceeded the maximum number of bytes per list-member",
                warning.output[0],
            )

    def test_extract_unquote_plus(self):
        self.assertEqual(
            self._extract("keykey=value%5Evalue"), {"keykey": "value^value"}
        )
        self.assertEqual(
            self._extract("key%23key=value%23value"),
            {"key#key": "value#value"},
        )

    def test_header_max_entries_skip_invalid_entry(self):
        with self.assertLogs(level=WARNING) as warning:
            self.assertEqual(
                self._extract(
                    ",".join(
                        [
                            (
                                f"key{index}=value{index}"
                                if index != 2
                                else (
                                    f"key{index}="
                                    f"value{'s' * (W3CBaggagePropagator._MAX_PAIR_LENGTH + 1)}"
                                )
                            )
                            for index in range(
                                W3CBaggagePropagator._MAX_PAIRS + 1
                            )
                        ]
                    )
                ),
                {
                    f"key{index}": f"value{index}"
                    for index in range(W3CBaggagePropagator._MAX_PAIRS + 1)
                    if index != 2
                },
            )
            self.assertIn(
                "exceeded the maximum number of list-members",
                warning.output[0],
            )

        with self.assertLogs(level=WARNING) as warning:
            self.assertEqual(
                self._extract(
                    ",".join(
                        [
                            (
                                f"key{index}=value{index}"
                                if index != 2
                                else f"key{index}xvalue{index}"
                            )
                            for index in range(
                                W3CBaggagePropagator._MAX_PAIRS + 1
                            )
                        ]
                    )
                ),
                {
                    f"key{index}": f"value{index}"
                    for index in range(W3CBaggagePropagator._MAX_PAIRS + 1)
                    if index != 2
                },
            )
            self.assertIn(
                "exceeded the maximum number of list-members",
                warning.output[0],
            )

    def test_inject_no_baggage_entries(self):
        values = {}
        output = self._inject(values)
        self.assertEqual(None, output)

    def test_inject_space_entries(self):
        self.assertEqual("key=val+ue", self._inject({"key": "val ue"}))

    def test_inject(self):
        values = {
            "key1": "val1",
            "key2": "val2",
        }
        output = self._inject(values)
        self.assertIn("key1=val1", output)
        self.assertIn("key2=val2", output)

    def test_inject_escaped_values(self):
        values = {
            "key1": "val1,val2",
            "key2": "val3=4",
        }
        output = self._inject(values)
        self.assertIn("key2=val3%3D4", output)

    def test_inject_non_string_values(self):
        values = {
            "key1": True,
            "key2": 123,
            "key3": 123.567,
        }
        output = self._inject(values)
        self.assertIn("key1=True", output)
        self.assertIn("key2=123", output)
        self.assertIn("key3=123.567", output)

    @patch("opentelemetry.baggage.propagation.get_all")
    @patch("opentelemetry.baggage.propagation._format_baggage")
    def test_fields(self, mock_format_baggage, mock_baggage):
        mock_setter = Mock()

        self.propagator.inject({}, setter=mock_setter)

        inject_fields = set()

        for mock_call in mock_setter.mock_calls:
            inject_fields.add(mock_call[1][1])

        self.assertEqual(inject_fields, self.propagator.fields)

    def test__format_baggage(self):
        self.assertEqual(
            _format_baggage({"key key": "value value"}), "key+key=value+value"
        )
        self.assertEqual(
            _format_baggage({"key/key": "value/value"}),
            "key%2Fkey=value%2Fvalue",
        )

    @patch("opentelemetry.baggage._BAGGAGE_KEY", new="abc")
    def test_inject_extract(self):
        carrier = {}

        context = set_baggage(
            "transaction", "string with spaces", context=get_current()
        )

        self.propagator.inject(carrier, context)

        context = self.propagator.extract(carrier)

        self.assertEqual(
            carrier, {"baggage": "transaction=string+with+spaces"}
        )

        self.assertEqual(
            context, {"abc": {"transaction": "string with spaces"}}
        )



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pickle
import unittest

from opentelemetry import trace


class TestSpanContext(unittest.TestCase):
    def test_span_context_pickle(self):
        """
        SpanContext needs to be pickleable to support multiprocessing
        so span can start as parent from the new spawned process
        """
        sc = trace.SpanContext(
            1,
            2,
            is_remote=False,
            trace_flags=trace.DEFAULT_TRACE_OPTIONS,
            trace_state=trace.DEFAULT_TRACE_STATE,
        )
        pickle_sc = pickle.loads(pickle.dumps(sc))
        self.assertEqual(sc.trace_id, pickle_sc.trace_id)
        self.assertEqual(sc.span_id, pickle_sc.span_id)

        invalid_sc = trace.SpanContext(
            9999999999999999999999999999999999999999999999999999999999999999999999999999,
            9,
            is_remote=False,
            trace_flags=trace.DEFAULT_TRACE_OPTIONS,
            trace_state=trace.DEFAULT_TRACE_STATE,
        )
        self.assertFalse(invalid_sc.is_valid)

    def test_trace_id_validity(self):
        trace_id_max_value = int("f" * 32, 16)
        span_id = 1

        # valid trace IDs
        sc = trace.SpanContext(trace_id_max_value, span_id, is_remote=False)
        self.assertTrue(sc.is_valid)

        sc = trace.SpanContext(1, span_id, is_remote=False)
        self.assertTrue(sc.is_valid)

        # invalid trace IDs
        sc = trace.SpanContext(0, span_id, is_remote=False)
        self.assertFalse(sc.is_valid)

        sc = trace.SpanContext(-1, span_id, is_remote=False)
        self.assertFalse(sc.is_valid)

        sc = trace.SpanContext(
            trace_id_max_value + 1, span_id, is_remote=False
        )
        self.assertFalse(sc.is_valid)

    def test_span_id_validity(self):
        span_id_max = int("f" * 16, 16)
        trace_id = 1

        # valid span IDs
        sc = trace.SpanContext(trace_id, span_id_max, is_remote=False)
        self.assertTrue(sc.is_valid)

        sc = trace.SpanContext(trace_id, 1, is_remote=False)
        self.assertTrue(sc.is_valid)

        # invalid span IDs
        sc = trace.SpanContext(trace_id, 0, is_remote=False)
        self.assertFalse(sc.is_valid)

        sc = trace.SpanContext(trace_id, -1, is_remote=False)
        self.assertFalse(sc.is_valid)

        sc = trace.SpanContext(trace_id, span_id_max + 1, is_remote=False)
        self.assertFalse(sc.is_valid)



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# type: ignore

from importlib import reload
from os import environ
from unittest import TestCase
from unittest.mock import Mock, patch

from opentelemetry import trace
from opentelemetry.baggage.propagation import W3CBaggagePropagator
from opentelemetry.context.context import Context
from opentelemetry.environment_variables import OTEL_PROPAGATORS
from opentelemetry.trace.propagation.tracecontext import (
    TraceContextTextMapPropagator,
)


class TestPropagators(TestCase):
    @patch("opentelemetry.propagators.composite.CompositePropagator")
    def test_default_composite_propagators(self, mock_compositehttppropagator):
        def test_propagators(propagators):
            propagators = {propagator.__class__ for propagator in propagators}

            self.assertEqual(len(propagators), 2)
            self.assertEqual(
                propagators,
                {TraceContextTextMapPropagator, W3CBaggagePropagator},
            )

        mock_compositehttppropagator.configure_mock(
            **{"side_effect": test_propagators}
        )

        # pylint: disable=import-outside-toplevel
        import opentelemetry.propagate

        reload(opentelemetry.propagate)

    @patch.dict(environ, {OTEL_PROPAGATORS: "a,  b,   c  "})
    @patch("opentelemetry.propagators.composite.CompositePropagator")
    @patch("opentelemetry.util._importlib_metadata.entry_points")
    def test_non_default_propagators(
        self, mock_entry_points, mock_compositehttppropagator
    ):
        mock_entry_points.configure_mock(
            **{
                "side_effect": [
                    [
                        Mock(
                            **{
                                "load.return_value": Mock(
                                    **{"return_value": "a"}
                                )
                            }
                        ),
                    ],
                    [
                        Mock(
                            **{
                                "load.return_value": Mock(
                                    **{"return_value": "b"}
                                )
                            }
                        )
                    ],
                    [
                        Mock(
                            **{
                                "load.return_value": Mock(
                                    **{"return_value": "c"}
                                )
                            }
                        )
                    ],
                ]
            }
        )

        def test_propagators(propagators):
            self.assertEqual(propagators, ["a", "b", "c"])

        mock_compositehttppropagator.configure_mock(
            **{"side_effect": test_propagators}
        )

        # pylint: disable=import-outside-toplevel
        import opentelemetry.propagate

        reload(opentelemetry.propagate)

    @patch.dict(
        environ, {OTEL_PROPAGATORS: "tracecontext , unknown , baggage"}
    )
    def test_composite_propagators_error(self):
        with self.assertRaises(ValueError) as cm:
            # pylint: disable=import-outside-toplevel
            import opentelemetry.propagate

            reload(opentelemetry.propagate)

        self.assertEqual(
            str(cm.exception),
            "Propagator unknown not found. It is either misspelled or not installed.",
        )


class TestTraceContextTextMapPropagator(TestCase):
    def setUp(self):
        self.propagator = TraceContextTextMapPropagator()

    def traceparent_helper(
        self,
        carrier,
    ):
        # We purposefully start with an empty context so we can test later if anything is added to it.
        initial_context = Context()

        context = self.propagator.extract(carrier, context=initial_context)
        self.assertIsNotNone(context)
        self.assertIsInstance(context, Context)

        return context

    def traceparent_helper_generator(
        self,
        version=0x00,
        trace_id=0x00000000000000000000000000000001,
        span_id=0x0000000000000001,
        trace_flags=0x00,
        suffix="",
    ):
        traceparent = f"{version:02x}-{trace_id:032x}-{span_id:016x}-{trace_flags:02x}{suffix}"
        carrier = {"traceparent": traceparent}
        return self.traceparent_helper(carrier)

    def valid_traceparent_helper(
        self,
        version=0x00,
        trace_id=0x00000000000000000000000000000001,
        span_id=0x0000000000000001,
        trace_flags=0x00,
        suffix="",
        assert_context_msg="A valid traceparent was provided, so the context should be non-empty.",
    ):
        context = self.traceparent_helper_generator(
            version=version,
            trace_id=trace_id,
            span_id=span_id,
            trace_flags=trace_flags,
            suffix=suffix,
        )

        self.assertNotEqual(
            context,
            Context(),
            assert_context_msg,
        )

        span = trace.get_current_span(context)
        self.assertIsNotNone(span)
        self.assertIsInstance(span, trace.span.Span)

        span_context = span.get_span_context()
        self.assertIsNotNone(span_context)
        self.assertIsInstance(span_context, trace.span.SpanContext)

        # Note: No version in SpanContext, it is only used locally in TraceContextTextMapPropagator
        self.assertEqual(span_context.trace_id, trace_id)
        self.assertEqual(span_context.span_id, span_id)
        self.assertEqual(span_context.trace_flags, trace_flags)

        self.assertIsInstance(span_context.trace_state, trace.TraceState)
        self.assertCountEqual(span_context.trace_state, [])
        self.assertEqual(span_context.is_remote, True)

        return context, span, span_context

    def invalid_traceparent_helper(
        self,
        version=0x00,
        trace_id=0x00000000000000000000000000000001,
        span_id=0x0000000000000001,
        trace_flags=0x00,
        suffix="",
        assert_context_msg="An invalid traceparent was provided, so the context should still be empty.",
    ):
        context = self.traceparent_helper_generator(
            version=version,
            trace_id=trace_id,
            span_id=span_id,
            trace_flags=trace_flags,
            suffix=suffix,
        )

        self.assertEqual(
            context,
            Context(),
            assert_context_msg,
        )

        return context

    def test_extract_nothing(self):
        context = self.traceparent_helper(carrier={})
        self.assertEqual(
            context,
            {},
            "We didn't provide a valid traceparent, so we should still have an empty Context.",
        )

    def test_extract_simple_traceparent(self):
        self.valid_traceparent_helper()

    # https://www.w3.org/TR/trace-context/#version
    def test_extract_version_forbidden_ff(self):
        self.invalid_traceparent_helper(
            version=0xFF,
            assert_context_msg="We provided ann invalid traceparent with a forbidden version=0xff, so the context should still be empty.",
        )

    # https://www.w3.org/TR/trace-context/#version-format
    def test_extract_version_00_with_unsupported_suffix(self):
        self.invalid_traceparent_helper(
            suffix="-f00",
            assert_context_msg="We provided an invalid traceparent with version=0x00 and suffix information which is not supported in this version, so the context should still be empty.",
        )

    # https://www.w3.org/TR/trace-context/#versioning-of-traceparent
    # See the parsing of the sampled bit of flags.
    def test_extract_future_version_with_future_suffix_data(self):
        self.valid_traceparent_helper(
            version=0x99,
            suffix="-f00",
            assert_context_msg="We provided a traceparent that is possibly valid in the future with version=0x99 and suffix information, so the context be non-empty.",
        )

    # https://www.w3.org/TR/trace-context/#trace-id
    def test_extract_trace_id_invalid_all_zeros(self):
        self.invalid_traceparent_helper(trace_id=0)

    # https://www.w3.org/TR/trace-context/#parent-id
    def test_extract_span_id_invalid_all_zeros(self):
        self.invalid_traceparent_helper(span_id=0)

    def test_extract_non_decimal_trace_flags(self):
        self.valid_traceparent_helper(trace_flags=0xA0)



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# type: ignore

from inspect import Signature, isabstract, signature
from unittest import TestCase

from opentelemetry.metrics import (
    Counter,
    Histogram,
    Instrument,
    Meter,
    NoOpCounter,
    NoOpHistogram,
    NoOpMeter,
    NoOpUpDownCounter,
    ObservableCounter,
    ObservableGauge,
    ObservableUpDownCounter,
    UpDownCounter,
    _Gauge,
)

# FIXME Test that the instrument methods can be called concurrently safely.


class ChildInstrument(Instrument):
    # pylint: disable=useless-parent-delegation
    def __init__(self, name, *args, unit="", description="", **kwargs):
        super().__init__(
            name, *args, unit=unit, description=description, **kwargs
        )


class TestCounter(TestCase):
    def test_create_counter(self):
        """
        Test that the Counter can be created with create_counter.
        """

        self.assertTrue(
            isinstance(NoOpMeter("name").create_counter("name"), Counter)
        )

    def test_api_counter_abstract(self):
        """
        Test that the API Counter is an abstract class.
        """

        self.assertTrue(isabstract(Counter))

    def test_create_counter_api(self):
        """
        Test that the API for creating a counter accepts the name of the instrument.
        Test that the API for creating a counter accepts the unit of the instrument.
        Test that the API for creating a counter accepts the description of the
        """

        create_counter_signature = signature(Meter.create_counter)
        self.assertIn("name", create_counter_signature.parameters.keys())
        self.assertIs(
            create_counter_signature.parameters["name"].default,
            Signature.empty,
        )

        create_counter_signature = signature(Meter.create_counter)
        self.assertIn("unit", create_counter_signature.parameters.keys())
        self.assertIs(create_counter_signature.parameters["unit"].default, "")

        create_counter_signature = signature(Meter.create_counter)
        self.assertIn(
            "description", create_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_counter_signature.parameters["description"].default, ""
        )

    def test_counter_add_method(self):
        """
        Test that the counter has an add method.
        Test that the add method returns None.
        Test that the add method accepts optional attributes.
        Test that the add method accepts the increment amount.
        Test that the add method accepts only positive amounts.
        """

        self.assertTrue(hasattr(Counter, "add"))

        self.assertIsNone(NoOpCounter("name").add(1))

        add_signature = signature(Counter.add)
        self.assertIn("attributes", add_signature.parameters.keys())
        self.assertIs(add_signature.parameters["attributes"].default, None)

        self.assertIn("amount", add_signature.parameters.keys())
        self.assertIs(
            add_signature.parameters["amount"].default, Signature.empty
        )


class TestObservableCounter(TestCase):
    def test_create_observable_counter(self):
        """
        Test that the ObservableCounter can be created with create_observable_counter.
        """

        def callback():
            yield

        self.assertTrue(
            isinstance(
                NoOpMeter("name").create_observable_counter(
                    "name", callbacks=[callback()]
                ),
                ObservableCounter,
            )
        )

    def test_api_observable_counter_abstract(self):
        """
        Test that the API ObservableCounter is an abstract class.
        """

        self.assertTrue(isabstract(ObservableCounter))

    def test_create_observable_counter_api(self):
        """
        Test that the API for creating a observable_counter accepts the name of the instrument.
        Test that the API for creating a observable_counter accepts a sequence of callbacks.
        Test that the API for creating a observable_counter accepts the unit of the instrument.
        Test that the API for creating a observable_counter accepts the description of the instrument
        """

        create_observable_counter_signature = signature(
            Meter.create_observable_counter
        )
        self.assertIn(
            "name", create_observable_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_counter_signature.parameters["name"].default,
            Signature.empty,
        )
        create_observable_counter_signature = signature(
            Meter.create_observable_counter
        )
        self.assertIn(
            "callbacks", create_observable_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_counter_signature.parameters[
                "callbacks"
            ].default,
            None,
        )
        create_observable_counter_signature = signature(
            Meter.create_observable_counter
        )
        self.assertIn(
            "unit", create_observable_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_counter_signature.parameters["unit"].default, ""
        )

        create_observable_counter_signature = signature(
            Meter.create_observable_counter
        )
        self.assertIn(
            "description",
            create_observable_counter_signature.parameters.keys(),
        )
        self.assertIs(
            create_observable_counter_signature.parameters[
                "description"
            ].default,
            "",
        )

    def test_observable_counter_generator(self):
        """
        Test that the API for creating a asynchronous counter accepts a generator.
        Test that the generator function reports iterable of measurements.
        Test that there is a way to pass state to the generator.
        Test that the instrument accepts positive measurements.
        Test that the instrument does not accept negative measurements.
        """

        create_observable_counter_signature = signature(
            Meter.create_observable_counter
        )
        self.assertIn(
            "callbacks", create_observable_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_counter_signature.parameters["name"].default,
            Signature.empty,
        )


class TestHistogram(TestCase):
    def test_create_histogram(self):
        """
        Test that the Histogram can be created with create_histogram.
        """

        self.assertTrue(
            isinstance(NoOpMeter("name").create_histogram("name"), Histogram)
        )

    def test_api_histogram_abstract(self):
        """
        Test that the API Histogram is an abstract class.
        """

        self.assertTrue(isabstract(Histogram))

    def test_create_histogram_api(self):
        """
        Test that the API for creating a histogram accepts the name of the instrument.
        Test that the API for creating a histogram accepts the unit of the instrument.
        Test that the API for creating a histogram accepts the description of the
        """

        create_histogram_signature = signature(Meter.create_histogram)
        self.assertIn("name", create_histogram_signature.parameters.keys())
        self.assertIs(
            create_histogram_signature.parameters["name"].default,
            Signature.empty,
        )

        create_histogram_signature = signature(Meter.create_histogram)
        self.assertIn("unit", create_histogram_signature.parameters.keys())
        self.assertIs(
            create_histogram_signature.parameters["unit"].default, ""
        )

        create_histogram_signature = signature(Meter.create_histogram)
        self.assertIn(
            "description", create_histogram_signature.parameters.keys()
        )
        self.assertIs(
            create_histogram_signature.parameters["description"].default, ""
        )

    def test_histogram_record_method(self):
        """
        Test that the histogram has an record method.
        Test that the record method returns None.
        Test that the record method accepts optional attributes.
        Test that the record method accepts the increment amount.
        Test that the record method returns None.
        """

        self.assertTrue(hasattr(Histogram, "record"))

        self.assertIsNone(NoOpHistogram("name").record(1))

        record_signature = signature(Histogram.record)
        self.assertIn("attributes", record_signature.parameters.keys())
        self.assertIs(record_signature.parameters["attributes"].default, None)

        self.assertIn("amount", record_signature.parameters.keys())
        self.assertIs(
            record_signature.parameters["amount"].default, Signature.empty
        )

        self.assertIsNone(NoOpHistogram("name").record(1))


class TestGauge(TestCase):
    def test_create_gauge(self):
        """
        Test that the Gauge can be created with create_gauge.
        """

        self.assertTrue(
            isinstance(NoOpMeter("name").create_gauge("name"), _Gauge)
        )

    def test_api_gauge_abstract(self):
        """
        Test that the API Gauge is an abstract class.
        """

        self.assertTrue(isabstract(_Gauge))

    def test_create_gauge_api(self):
        """
        Test that the API for creating a gauge accepts the name of the instrument.
        Test that the API for creating a gauge accepts a sequence of callbacks.
        Test that the API for creating a gauge accepts the unit of the instrument.
        Test that the API for creating a gauge accepts the description of the instrument
        """

        create_gauge_signature = signature(Meter.create_gauge)
        self.assertIn("name", create_gauge_signature.parameters.keys())
        self.assertIs(
            create_gauge_signature.parameters["name"].default,
            Signature.empty,
        )
        create_gauge_signature = signature(Meter.create_gauge)
        create_gauge_signature = signature(Meter.create_gauge)
        self.assertIn("unit", create_gauge_signature.parameters.keys())
        self.assertIs(create_gauge_signature.parameters["unit"].default, "")

        create_gauge_signature = signature(Meter.create_gauge)
        self.assertIn("description", create_gauge_signature.parameters.keys())
        self.assertIs(
            create_gauge_signature.parameters["description"].default,
            "",
        )


class TestObservableGauge(TestCase):
    def test_create_observable_gauge(self):
        """
        Test that the ObservableGauge can be created with create_observable_gauge.
        """

        def callback():
            yield

        self.assertTrue(
            isinstance(
                NoOpMeter("name").create_observable_gauge(
                    "name", [callback()]
                ),
                ObservableGauge,
            )
        )

    def test_api_observable_gauge_abstract(self):
        """
        Test that the API ObservableGauge is an abstract class.
        """

        self.assertTrue(isabstract(ObservableGauge))

    def test_create_observable_gauge_api(self):
        """
        Test that the API for creating a observable_gauge accepts the name of the instrument.
        Test that the API for creating a observable_gauge accepts a sequence of callbacks.
        Test that the API for creating a observable_gauge accepts the unit of the instrument.
        Test that the API for creating a observable_gauge accepts the description of the instrument
        """

        create_observable_gauge_signature = signature(
            Meter.create_observable_gauge
        )
        self.assertIn(
            "name", create_observable_gauge_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_gauge_signature.parameters["name"].default,
            Signature.empty,
        )
        create_observable_gauge_signature = signature(
            Meter.create_observable_gauge
        )
        self.assertIn(
            "callbacks", create_observable_gauge_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_gauge_signature.parameters["callbacks"].default,
            None,
        )
        create_observable_gauge_signature = signature(
            Meter.create_observable_gauge
        )
        self.assertIn(
            "unit", create_observable_gauge_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_gauge_signature.parameters["unit"].default, ""
        )

        create_observable_gauge_signature = signature(
            Meter.create_observable_gauge
        )
        self.assertIn(
            "description", create_observable_gauge_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_gauge_signature.parameters[
                "description"
            ].default,
            "",
        )

    def test_observable_gauge_callback(self):
        """
        Test that the API for creating a asynchronous gauge accepts a sequence of callbacks.
        Test that the callback function reports measurements.
        Test that there is a way to pass state to the callback.
        """

        create_observable_gauge_signature = signature(
            Meter.create_observable_gauge
        )
        self.assertIn(
            "callbacks", create_observable_gauge_signature.parameters.keys()
        )
        self.assertIs(
            create_observable_gauge_signature.parameters["name"].default,
            Signature.empty,
        )


class TestUpDownCounter(TestCase):
    def test_create_up_down_counter(self):
        """
        Test that the UpDownCounter can be created with create_up_down_counter.
        """

        self.assertTrue(
            isinstance(
                NoOpMeter("name").create_up_down_counter("name"),
                UpDownCounter,
            )
        )

    def test_api_up_down_counter_abstract(self):
        """
        Test that the API UpDownCounter is an abstract class.
        """

        self.assertTrue(isabstract(UpDownCounter))

    def test_create_up_down_counter_api(self):
        """
        Test that the API for creating a up_down_counter accepts the name of the instrument.
        Test that the API for creating a up_down_counter accepts the unit of the instrument.
        Test that the API for creating a up_down_counter accepts the description of the
        """

        create_up_down_counter_signature = signature(
            Meter.create_up_down_counter
        )
        self.assertIn(
            "name", create_up_down_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_up_down_counter_signature.parameters["name"].default,
            Signature.empty,
        )

        create_up_down_counter_signature = signature(
            Meter.create_up_down_counter
        )
        self.assertIn(
            "unit", create_up_down_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_up_down_counter_signature.parameters["unit"].default, ""
        )

        create_up_down_counter_signature = signature(
            Meter.create_up_down_counter
        )
        self.assertIn(
            "description", create_up_down_counter_signature.parameters.keys()
        )
        self.assertIs(
            create_up_down_counter_signature.parameters["description"].default,
            "",
        )

    def test_up_down_counter_add_method(self):
        """
        Test that the up_down_counter has an add method.
        Test that the add method returns None.
        Test that the add method accepts optional attributes.
        Test that the add method accepts the increment or decrement amount.
        Test that the add method accepts positive and negative amounts.
        """

        self.assertTrue(hasattr(UpDownCounter, "add"))

        self.assertIsNone(NoOpUpDownCounter("name").add(1))

        add_signature = signature(UpDownCounter.add)
        self.assertIn("attributes", add_signature.parameters.keys())
        self.assertIs(add_signature.parameters["attributes"].default, None)

        self.assertIn("amount", add_signature.parameters.keys())
        self.assertIs(
            add_signature.parameters["amount"].default, Signature.empty
        )


class TestObservableUpDownCounter(TestCase):
    # pylint: disable=protected-access
    def test_create_observable_up_down_counter(self):
        """
        Test that the ObservableUpDownCounter can be created with create_observable_up_down_counter.
        """

        def callback():
            yield

        self.assertTrue(
            isinstance(
                NoOpMeter("name").create_observable_up_down_counter(
                    "name", [callback()]
                ),
                ObservableUpDownCounter,
            )
        )

    def test_api_observable_up_down_counter_abstract(self):
        """
        Test that the API ObservableUpDownCounter is an abstract class.
        """

        self.assertTrue(isabstract(ObservableUpDownCounter))

    def test_create_observable_up_down_counter_api(self):
        """
        Test that the API for creating a observable_up_down_counter accepts the name of the instrument.
        Test that the API for creating a observable_up_down_counter accepts a sequence of callbacks.
        Test that the API for creating a observable_up_down_counter accepts the unit of the instrument.
        Test that the API for creating a observable_up_down_counter accepts the description of the instrument
        """

        create_observable_up_down_counter_signature = signature(
            Meter.create_observable_up_down_counter
        )
        self.assertIn(
            "name",
            create_observable_up_down_counter_signature.parameters.keys(),
        )
        self.assertIs(
            create_observable_up_down_counter_signature.parameters[
                "name"
            ].default,
            Signature.empty,
        )
        create_observable_up_down_counter_signature = signature(
            Meter.create_observable_up_down_counter
        )
        self.assertIn(
            "callbacks",
            create_observable_up_down_counter_signature.parameters.keys(),
        )
        self.assertIs(
            create_observable_up_down_counter_signature.parameters[
                "callbacks"
            ].default,
            None,
        )
        create_observable_up_down_counter_signature = signature(
            Meter.create_observable_up_down_counter
        )
        self.assertIn(
            "unit",
            create_observable_up_down_counter_signature.parameters.keys(),
        )
        self.assertIs(
            create_observable_up_down_counter_signature.parameters[
                "unit"
            ].default,
            "",
        )

        create_observable_up_down_counter_signature = signature(
            Meter.create_observable_up_down_counter
        )
        self.assertIn(
            "description",
            create_observable_up_down_counter_signature.parameters.keys(),
        )
        self.assertIs(
            create_observable_up_down_counter_signature.parameters[
                "description"
            ].default,
            "",
        )

    def test_observable_up_down_counter_callback(self):
        """
        Test that the API for creating a asynchronous up_down_counter accepts a sequence of callbacks.
        Test that the callback function reports measurements.
        Test that there is a way to pass state to the callback.
        Test that the instrument accepts positive and negative values.
        """

        create_observable_up_down_counter_signature = signature(
            Meter.create_observable_up_down_counter
        )
        self.assertIn(
            "callbacks",
            create_observable_up_down_counter_signature.parameters.keys(),
        )
        self.assertIs(
            create_observable_up_down_counter_signature.parameters[
                "name"
            ].default,
            Signature.empty,
        )

    def test_name_check(self):
        instrument = ChildInstrument("name")

        self.assertEqual(
            instrument._check_name_unit_description(
                "a" * 255, "unit", "description"
            )["name"],
            "a" * 255,
        )
        self.assertEqual(
            instrument._check_name_unit_description(
                "a.", "unit", "description"
            )["name"],
            "a.",
        )
        self.assertEqual(
            instrument._check_name_unit_description(
                "a-", "unit", "description"
            )["name"],
            "a-",
        )
        self.assertEqual(
            instrument._check_name_unit_description(
                "a_", "unit", "description"
            )["name"],
            "a_",
        )
        self.assertEqual(
            instrument._check_name_unit_description(
                "a/", "unit", "description"
            )["name"],
            "a/",
        )

        # the old max length
        self.assertIsNotNone(
            instrument._check_name_unit_description(
                "a" * 64, "unit", "description"
            )["name"]
        )
        self.assertIsNone(
            instrument._check_name_unit_description(
                "a" * 256, "unit", "description"
            )["name"]
        )
        self.assertIsNone(
            instrument._check_name_unit_description(
                "Ñ", "unit", "description"
            )["name"]
        )
        self.assertIsNone(
            instrument._check_name_unit_description(
                "_a", "unit", "description"
            )["name"]
        )
        self.assertIsNone(
            instrument._check_name_unit_description(
                "1a", "unit", "description"
            )["name"]
        )
        self.assertIsNone(
            instrument._check_name_unit_description("", "unit", "description")[
                "name"
            ]
        )

    def test_unit_check(self):
        instrument = ChildInstrument("name")

        self.assertEqual(
            instrument._check_name_unit_description(
                "name", "a" * 63, "description"
            )["unit"],
            "a" * 63,
        )
        self.assertEqual(
            instrument._check_name_unit_description(
                "name", "{a}", "description"
            )["unit"],
            "{a}",
        )

        self.assertIsNone(
            instrument._check_name_unit_description(
                "name", "a" * 64, "description"
            )["unit"]
        )
        self.assertIsNone(
            instrument._check_name_unit_description(
                "name", "Ñ", "description"
            )["unit"]
        )
        self.assertEqual(
            instrument._check_name_unit_description(
                "name", None, "description"
            )["unit"],
            "",
        )

    def test_description_check(self):
        instrument = ChildInstrument("name")

        self.assertEqual(
            instrument._check_name_unit_description(
                "name", "unit", "description"
            )["description"],
            "description",
        )
        self.assertEqual(
            instrument._check_name_unit_description("name", "unit", None)[
                "description"
            ],
            "",
        )



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=W0212,W0222,W0221
import typing
import unittest

from opentelemetry import trace
from opentelemetry.test.globals_test import TraceGlobalsTest
from opentelemetry.trace.span import (
    INVALID_SPAN_CONTEXT,
    NonRecordingSpan,
    Span,
)
from opentelemetry.util._decorator import _agnosticcontextmanager
from opentelemetry.util.types import Attributes


class TestProvider(trace.NoOpTracerProvider):
    def get_tracer(
        self,
        instrumenting_module_name: str,
        instrumenting_library_version: typing.Optional[str] = None,
        schema_url: typing.Optional[str] = None,
        attributes: typing.Optional[Attributes] = None,
    ) -> trace.Tracer:
        return TestTracer()


class TestTracer(trace.NoOpTracer):
    def start_span(self, *args, **kwargs):
        return SpanTest(INVALID_SPAN_CONTEXT)

    @_agnosticcontextmanager  # pylint: disable=protected-access
    def start_as_current_span(self, *args, **kwargs):  # type: ignore
        with trace.use_span(self.start_span(*args, **kwargs)) as span:  # type: ignore
            yield span


class SpanTest(NonRecordingSpan):
    pass


class TestProxy(TraceGlobalsTest, unittest.TestCase):
    def test_proxy_tracer(self):
        provider = trace.get_tracer_provider()
        # proxy provider
        self.assertIsInstance(provider, trace.ProxyTracerProvider)

        # provider returns proxy tracer
        tracer = provider.get_tracer("proxy-test")
        self.assertIsInstance(tracer, trace.ProxyTracer)

        with tracer.start_span("span1") as span:
            self.assertIsInstance(span, trace.NonRecordingSpan)

        with tracer.start_as_current_span("span2") as span:
            self.assertIsInstance(span, trace.NonRecordingSpan)

        # set a real provider
        trace.set_tracer_provider(TestProvider())

        # get_tracer_provider() now returns the real provider
        self.assertIsInstance(trace.get_tracer_provider(), TestProvider)

        # tracer provider now returns real instance
        self.assertIsInstance(trace.get_tracer_provider(), TestProvider)

        # references to the old provider still work but return real tracer now
        real_tracer = provider.get_tracer("proxy-test")
        self.assertIsInstance(real_tracer, TestTracer)

        # reference to old proxy tracer now delegates to a real tracer and
        # creates real spans
        with tracer.start_span("") as span:
            self.assertIsInstance(span, SpanTest)

    def test_late_config(self):
        # get a tracer and instrument a function as we would at the
        # root of a module
        tracer = trace.get_tracer("test")

        @tracer.start_as_current_span("span")
        def my_function() -> Span:
            return trace.get_current_span()

        # call function before configuring tracing provider, should
        # return INVALID_SPAN from the NoOpTracer
        self.assertEqual(my_function(), trace.INVALID_SPAN)

        # configure tracing provider
        trace.set_tracer_provider(TestProvider())
        # call function again, we should now be getting a TestSpan
        self.assertIsInstance(my_function(), SpanTest)



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# type: ignore

import unittest

from opentelemetry.propagators.textmap import DefaultGetter


class TestDefaultGetter(unittest.TestCase):
    def test_get_none(self):
        getter = DefaultGetter()
        carrier = {}
        val = getter.get(carrier, "test")
        self.assertIsNone(val)

    def test_get_str(self):
        getter = DefaultGetter()
        carrier = {"test": "val"}
        val = getter.get(carrier, "test")
        self.assertEqual(val, ["val"])

    def test_get_iter(self):
        getter = DefaultGetter()
        carrier = {"test": ["val"]}
        val = getter.get(carrier, "test")
        self.assertEqual(val, ["val"])

    def test_keys(self):
        getter = DefaultGetter()
        keys = getter.keys({"test": "val"})
        self.assertEqual(keys, ["test"])
